@@ -1,9 +1,22 @@
 #ifndef __VSCAN_FILEACCESSLOG_H_
 #define __VSCAN_FILEACCESSLOG_H_
 
+/* maximum number of entries */
 #define MAX_LRUFILES 100
+/* invalidate entry after x seconds */
 #define LRUFILES_INVALIDATE_TIME 10  
 
+
+/* actions to be performed */
+
+/* file needs to be scanned */
+#define VSCAN_LRU_SCAN_FILE     1
+/* deny access to file */
+#define VSCAN_LRU_DENY_ACCESS  -1
+/* grant access without scanning */
+#define VSCAN_LRU_GRANT_ACCESS  0
+
+
 struct lrufiles_struct {
 	struct lrufiles_struct *prev, *next;
 	pstring fname;		/* the file name */
