@@ -10,7 +10,8 @@
 #include "vscan-message.h"
 #include "vscan-quarantine.h"
 #include "vscan-filetype.h"
-
+#include "vscan-parameter.h"
+#include "vscan-scan.h"
 
 #define CLIENT_IP_SIZE 18
 
@@ -57,8 +58,16 @@
 #define SYSLOG_PRIORITY_ALERT   LOG_ERR
 #endif
 
+/* end configuration section */
 
 
+#if (SMB_VFS_INTERFACE_VERSION < 6)
+ #if (SAMBA_VERSION_MAJOR==3) || (SAMBA_VERSION_RELEASE>=4)
+ #define PROTOTYPE_CONST const
+ #else
+ #define PROTOTYPE_CONST
+ #endif
+#endif
 
 
 
