@@ -687,13 +687,20 @@ static int isadir(const char* path,char flags)
   }
 }
 
+/*getDate() - returns a pointer to 16-char string(16th is 0),
+  representing current date and time*/
 static char* getDate()
 {
   char* date;
   time_t t;
   time(&t);
   date=(char*)malloc(16);
-  strncpy(date,ctime(&t)+4,15);
+/*  strncpy(date,ctime(&t)+4,15);
+  strncpy is re-defined as safe_strcpy by Samba. This function
+  checks the length of date string (which is 16), but only 15 chars
+  are copied. So, the "ERROR: string overflow by 1 (16 - 15) in safe_strcpy"
+  is logged. reniar, 03.07.2004 */
+  memmove(date,ctime(&t)+4,15); 
   date[15]=0;
   return date;
 }
