@@ -1,10 +1,10 @@
 /*
- * $Id: vscan-sophos.c,v 1.47.2.2 2003/09/12 07:12:16 reniar Exp $
+ * $Id: vscan-sophos.c,v 1.47.2.5 2005/02/23 02:49:40 reniar Exp $
  * 
  * virusscanning VFS module for samba.  Log infected files via syslog
  * facility and block access using Sophie.
  *
- * Copyright (C) Rainer Link, 2001-2003
+ * Copyright (C) Rainer Link, 2001-2004
  *               OpenAntiVirus.org <rainer@openantivirus.org>
  * Copyright (C) Stefan (metze) Metzmacher, 2003
  *               <metze@metzemix.de>
@@ -45,48 +45,15 @@
 
 #define VSCAN_MODULE_STR "vscan-sophos"
 
-fstring config_file;            /* location of config file, either
-                                   PARAMCONF or as set via vfs options
-                                */
+vscan_config_struct vscan_config; /* contains the vscan module configuration */
 
-ssize_t max_size;          	/* do not scan files greater than max_size
-                                   if max_size = 0, scan any file
-                                */
 
 BOOL verbose_file_logging;  	/* log ever file access */
-
-BOOL scan_on_open;         	/* scan a file before it is opened
-                                   Defaults to True
-                                */
-
-BOOL scan_on_close;        	/* scan a new file put on share or
-                                   if file was modified
-                                   Defaults to False
-                                */
-
-BOOL deny_access_on_error;	/* if using SAVI fails, should access to any
-				   file be denied? */
-
-BOOL deny_access_on_minor_error; /* if daemon returns non-critical error,
-                                    should access to the file be denied? */
-
 BOOL send_warning_message;	/* send a warning message using the windows
 				   messenger service? */
 
-
 fstring	sophie_socket_name;	/* name of Sophie socket */
 
-fstring quarantine_dir;	/* directory for infected files */
-fstring quarantine_prefix;	/* prefix    for infected files */
-
-enum infected_file_action_enum infected_file_action; /* what to do with infected files;
-                                                        defaults to quarantine */
-
-int max_lrufiles;               /* specified the maximum entries in lrufiles list */
-time_t lrufiles_invalidate_time; /* specified the time in seconds after the lifetime
-                                    of an entry is expired and entry will be invalidated */
-pstring exclude_file_types;	/* list of file types which should be excluded from scanning */
-
 
 /* module version */
 static const char module_id[]=VSCAN_MODULE_STR" "SAMBA_VSCAN_VERSION_STR;
@@ -95,60 +62,13 @@ static const char module_id[]=VSCAN_MODULE_STR" "SAMBA_VSCAN_VERSION_STR;
 
 static BOOL do_parameter(const char *param, const char *value)
 {
-	if ( StrCaseCmp("max file size", param) == 0 ) {
-		max_size = atoi(value);
-		DEBUG(3, ("max file size is: %d\n", max_size));
-	} else if ( StrCaseCmp("verbose file logging", param) == 0 ) {
-		set_boolean(&verbose_file_logging, value);
-		DEBUG(3, ("verbose file logging is: %d\n", verbose_file_logging));	
-	} else if ( StrCaseCmp("scan on open", param) == 0 ) {
-		set_boolean(&scan_on_open, value);
-		DEBUG(3, ("scan on open: %d\n", scan_on_open));
-	} else if ( StrCaseCmp("scan on close", param) == 0 ) {
-		set_boolean(&scan_on_close, value);
-		DEBUG(3, ("scan on close is: %d\n", scan_on_close));
-	} else if ( StrCaseCmp("deny access on error", param) == 0 ) {
-		set_boolean(&deny_access_on_error, value);
-		DEBUG(3, ("deny access on error is: %d\n", deny_access_on_error));
-	} else if ( StrCaseCmp("deny access on minor error", param) == 0 ) {
-		set_boolean(&deny_access_on_minor_error, value);
-		DEBUG(3, ("deny access on minor error is: %d\n", deny_access_on_minor_error));
-	} else if ( StrCaseCmp("send warning message", param) == 0 ) {
-		set_boolean(&send_warning_message, value);
-		DEBUG(3, ("send warning message is: %d\n", send_warning_message));
-
-        } else if ( StrCaseCmp("infected file action", param) == 0 ) {
-		if (StrCaseCmp("quarantine", value) == 0) {
-			infected_file_action = INFECTED_QUARANTINE;
-		} else if (StrCaseCmp("delete", value) == 0) {
-			infected_file_action = INFECTED_DELETE;
-		} else if (StrCaseCmp("nothing", value) == 0) {
-			infected_file_action = INFECTED_DO_NOTHING;
-		} else {
-			DEBUG(2, ("samba-vscan: badly formed infected file action in configuration file, parameter %s\n", value));
+       if ( do_common_parameter(&vscan_config, param, value) == False ) {
+                /* parse VFS module specific configuration values */
+		if ( StrCaseCmp("sophie socket name", param) == 0) {
+			fstrcpy(sophie_socket_name, value);
+			DEBUG(3, ("sophie socket name is %s\n", sophie_socket_name));
 		}
-                DEBUG(3, ("infected file action is: %d\n", infected_file_action));
-        } else if ( StrCaseCmp("quarantine directory", param) == 0 ) {
-                fstrcpy(quarantine_dir, value);
-                DEBUG(3, ("quarantine directory is: %s\n", quarantine_dir));
-        } else if ( StrCaseCmp("quarantine prefix", param) == 0 ) {
-                fstrcpy(quarantine_prefix, value);
-                DEBUG(3, ("quarantine prefix is: %s\n", quarantine_prefix));
-        } else if ( StrCaseCmp("max lru files entries", param) == 0 ) {
-                max_lrufiles = atoi(value);
-                DEBUG(3, ("max lru files entries is: %d\n", max_lrufiles));
-        } else if ( StrCaseCmp("lru file entry lifetime", param) == 0 ) {
-                lrufiles_invalidate_time = atol(value);
-                DEBUG(3, ("lru file entry lifetime is: %li\n", (long)lrufiles_invalidate_time));
-
-	} else if ( StrCaseCmp("sophie socket name", param) == 0) {
-		fstrcpy(sophie_socket_name, value);
-		DEBUG(3, ("sophie socket name is %s\n", sophie_socket_name));
-	} else if ( StrCaseCmp("exclude file types", param) == 0 ) {
-		pstrcpy(exclude_file_types, value);
-		DEBUG(3, ("Exclude list is: %s\n", exclude_file_types));
-	} else
-                DEBUG(3, ("unknown parameter: %s\n", param));
+	}
 
         return True;
 }
@@ -170,6 +90,10 @@ static int vscan_connect(vfs_handle_struct *handle, connection_struct *conn, con
 static int vscan_connect(struct connection_struct *conn, PROTOTYPE_CONST char *svc, PROTOTYPE_CONST char *user)
 #endif
 {
+        fstring config_file;            /* location of config file, either
+                                           PARAMCONF or as set via vfs options
+                                        */
+
 	#if (SAMBA_VERSION_MAJOR==2 && SAMBA_VERSION_RELEASE>=4) || SAMBA_VERSION_MAJOR==3
 	 #if !(SMB_VFS_INTERFACE_VERSION >= 6)
           pstring opts_str;
@@ -185,83 +109,36 @@ static int vscan_connect(struct connection_struct *conn, PROTOTYPE_CONST char *s
 	/* set default value for configuration files */
 	fstrcpy(config_file, PARAMCONF);
 
-	/* set default value for max file size */
-	max_size = VSCAN_MAX_SIZE;
-
-	/* set default value for file logging */
-	verbose_file_logging = VSCAN_VERBOSE_FILE_LOGGING;
-
-	/* set default value for scan on open() */
-	scan_on_open = VSCAN_SCAN_ON_OPEN;
-
-	/* set default value for scan on close() */
-	scan_on_close = VSCAN_SCAN_ON_CLOSE;
-
-	/* set default value for deny access on error */
-	deny_access_on_error = VSCAN_DENY_ACCESS_ON_ERROR;
-
-	/* set default value for deny access on minor error */
-	deny_access_on_minor_error = VSCAN_DENY_ACCESS_ON_MINOR_ERROR;
- 
-	/* set default value for send warning message */
-	send_warning_message = VSCAN_SEND_WARNING_MESSAGE;
+        /* set default values */
+        set_common_default_settings(&vscan_config);
 
 	/* name of Sophie socket */
 	fstrcpy(sophie_socket_name, SOPHIE_SOCKET_NAME);
 
-        /* set default value for maximum lrufile entries */
-        max_lrufiles = VSCAN_MAX_LRUFILES;
-
-        /* time after an entry is considered as expired */
-        lrufiles_invalidate_time = VSCAN_LRUFILES_INVALIDATE_TIME;
-
-	/* file type exclude ist */
-	pstrcpy(exclude_file_types, VSCAN_FT_EXCLUDE_LIST);
-
 	vscan_syslog("INFO: connect to service %s by user %s", 
 	       svc, user);
 
 	#if (SAMBA_VERSION_MAJOR==2 && SAMBA_VERSION_RELEASE>=4) || SAMBA_VERSION_MAJOR==3
-	 #if (SMB_VFS_INTERFACE_VERSION >= 6)
-	  fstrcpy(config_file, lp_parm_const_string(SNUM(conn),VSCAN_MODULE_STR,"config-file",PARAMCONF));
-	 #else
-          pstrcpy(opts_str, (const char*) lp_vfs_options(SNUM(conn)));
-          if( !*opts_str ) {
-                DEBUG(3, ("samba-vscan: no configuration file set - using default value (%s).\n", lp_vfs_options(SNUM(conn))));
-          } else {
-                p = opts_str;
-                if ( next_token(&p, config_file, "=", sizeof(config_file)) ) {
-                        trim_string(config_file, " ", " ");
-                        if ( !strequal("config-file", config_file) ) {
-                                DEBUG(3, ("samba-vscan - connect: options %s is not config-file\n", config_file));
-                                /* setting default value */
-                                fstrcpy(config_file, PARAMCONF);
+          fstrcpy(config_file, get_configuration_file(conn, VSCAN_MODULE_STR, PARAMCONF));
+          DEBUG(3, ("configuration file is: %s\n", config_file));
 
-                        } else {
-                                if ( !next_token(&p, config_file," \n",sizeof(config_file)) ) {
-                                        DEBUG(3, ("samba-vscan - connect: no option after config-file=\n"));
-                                        /* setting default value */
-                                        fstrcpy(config_file, PARAMCONF);
-                                } else {
-                                        trim_string(config_file, " ", " ");
-                                        DEBUG(3, ("samba-vscan - connect: config file name is %s\n", config_file));
-                                }
-                        }
-                }
-          }
-	  #endif /*  #if (SMB_VFS_INTERFACE_VERSION >= 6)*/
           retval = pm_process(config_file, do_section, do_parameter);
           DEBUG(10, ("pm_process returned %d\n", retval));
+
+         /* FIXME: this is lame! */
+          verbose_file_logging = vscan_config.common.verbose_file_logging;
+          send_warning_message = vscan_config.common.send_warning_message;
+
 	  if (!retval) vscan_syslog("ERROR: could not parse configuration file '%s'. File not found or not read-able. Using compiled-in defaults", config_file);
 	#endif
 
         /* initialise lrufiles list */
         DEBUG(5, ("init lrufiles list\n"));
-        lrufiles_init(max_lrufiles, lrufiles_invalidate_time);
+        lrufiles_init(vscan_config.common.max_lrufiles, vscan_config.common.lrufiles_invalidate_time);
 
 	/* initialise filetype */
 	DEBUG(5, ("init file type\n"));
-	filetype_init(0, exclude_file_types);
+	filetype_init(0, vscan_config.common.exclude_file_types);
 
 
 
@@ -315,7 +192,7 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 	pstrcat(filepath, fname);
 
         /* scan files while opening? */
-        if ( !scan_on_open ) {
+        if ( !vscan_config.common.scan_on_open ) {
                 DEBUG(3, ("samba-vscan - open: File '%s' not scanned as scan_on_open is not set\n", fname));
 #if (SMB_VFS_INTERFACE_VERSION >= 6)
 		return SMB_VFS_NEXT_OPEN(handle, conn, fname, flags, mode);
@@ -337,7 +214,7 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 #else
 		return default_vfs_ops.open(conn, fname, flags, mode);
 #endif
-	else if ( ( stat_buf.st_size > max_size ) && ( max_size > 0 ) ) /* file is too large */
+	else if ( ( stat_buf.st_size > vscan_config.common.max_size ) && ( vscan_config.common.max_size > 0 ) ) /* file is too large */
 		vscan_syslog("INFO: File %s is larger than specified maximum file size! Not scanned!", fname);
 	else if ( stat_buf.st_size == 0 ) /* do not scan empty files */
 #if (SMB_VFS_INTERFACE_VERSION >= 6)
@@ -345,8 +222,8 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 #else
 		return default_vfs_ops.open(conn, fname, flags, mode);
 #endif
-	else if ( filetype_skipscan(filepath) == 1 ) {
-		if ( verbose_file_logging )
+	else if ( filetype_skipscan(filepath) == VSCAN_FT_SKIP_SCAN ) {
+		if ( vscan_config.common.verbose_file_logging )
 			vscan_syslog("File '%s' not scanned as file type is on exclude list", filepath);
 #if (SMB_VFS_INTERFACE_VERSION >= 6)
                 return SMB_VFS_NEXT_OPEN(handle, conn, fname, flags, mode);
@@ -358,7 +235,7 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 		/* open socket */
 		sockfd = vscan_sophos_init();
 
-                if ( sockfd == -1 && deny_access_on_error ) {
+                if ( sockfd == -1 && vscan_config.common.deny_access_on_error ) {
                         /* an error occured - can not communicate to daemon - deny access */
                         vscan_syslog("ERROR: can not communicate to daemon - access denied");
                         errno = EACCES;
@@ -368,10 +245,10 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 
                         /* must file actually be scanned? */
                         must_be_checked = lrufiles_must_be_checked(filepath, stat_buf.st_mtime);
-                        if ( must_be_checked == -1 ) {
+                        if ( must_be_checked == VSCAN_LRU_DENY_ACCESS ) {
                                 /* file has already been checked and marked as infected */
                                 /* deny access */
-                                if ( verbose_file_logging )
+                                if ( vscan_config.common.verbose_file_logging )
                                         vscan_syslog("File '%s' has already been scanned and marked as infected. Not scanned any more. Access denied", filepath);
 			
 				/* close socket */
@@ -380,9 +257,9 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 				/* deny access */
                                 errno = EACCES;
                                 return -1;
-                        } else if ( must_be_checked == 0 )  {
+                        } else if ( must_be_checked == VSCAN_LRU_GRANT_ACCESS )  {
                                 /* file has already been checked, not marked as infected and not modified */
-                                if ( verbose_file_logging )
+                                if ( vscan_config.common.verbose_file_logging )
                                         vscan_syslog("File '%s' has already been scanned, not marked as infected and not modified. Not scanned anymore. Access granted", filepath);
 
                                 /* close socket */
@@ -399,7 +276,7 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 
 			/* scan file */
 			retval = vscan_sophos_scanfile(sockfd, filepath, client_ip);
-			if ( retval == -2 && deny_access_on_minor_error ) {
+			if ( retval == VSCAN_SCAN_MINOR_ERROR && vscan_config.common.deny_access_on_minor_error ) {
 				/* a minor error occured - deny access */
 				vscan_syslog("ERROR: daemon failed with a minor error - access to file %s denied", fname);
 				vscan_sophos_end(sockfd);
@@ -410,7 +287,7 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
                                 /* deny access */
 				errno = EACCES;
 				return -1;
-                        } else if ( retval == -1 && deny_access_on_error ) {
+                        } else if ( retval == VSCAN_SCAN_ERROR && vscan_config.common.deny_access_on_error ) {
 			/* an error occured - can not communicate to daemon - deny access */
                                 vscan_syslog("ERROR: can not communicate to Sophie - access to file %s denied", fname);
 				vscan_sophos_end(sockfd);
@@ -422,16 +299,16 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 
                                 errno = EACCES;
                                 return -1;
-			} else if ( retval == 1 ) {
+			} else if ( retval == VSCAN_SCAN_VIRUS_FOUND ) {
 				/* virus found */
 				vscan_sophos_end(sockfd);
 
 				/* do action ... */
 				
 #if (SMB_VFS_INTERFACE_VERSION >= 6)
-				rc = vscan_do_infected_file_action(handle, conn, filepath, quarantine_dir, quarantine_prefix, infected_file_action);
+				rc = vscan_do_infected_file_action(handle, conn, filepath, vscan_config.common.quarantine_dir, vscan_config.common.quarantine_prefix, vscan_config.common.infected_file_action);
 #else
-				rc = vscan_do_infected_file_action(&default_vfs_ops, conn, filepath, quarantine_dir, quarantine_prefix, infected_file_action);
+				rc = vscan_do_infected_file_action(&default_vfs_ops, conn, filepath, vscan_config.common.quarantine_dir, vscan_config.common.quarantine_prefix, vscan_config.common.infected_file_action);
 #endif
 
                                 /* add/update file. mark file as infected! */
@@ -440,7 +317,7 @@ static int vscan_open(struct connection_struct *conn, PROTOTYPE_CONST char *fnam
 				/* virus found, deny acces */
 				errno = EACCES; 
 				return -1;
-                        } else if ( retval == 0 ) {
+                        } else if ( retval == VSCAN_SCAN_OK ) {
                                 /* file is clean, add to lrufiles */
                                 lrufiles_add(filepath, stat_buf.st_mtime, False);
                         }
@@ -474,7 +351,7 @@ static int vscan_close(struct files_struct *fsp, int fd)
         retval = default_vfs_ops.close(fsp, fd);
 #endif
 
-        if ( !scan_on_close ) {
+        if ( !vscan_config.common.scan_on_close ) {
                 DEBUG(3, ("samba-vscan - close: File '%s' not scanned as scan_on_close is not set\n", fsp->fsp_name));
                 return retval;
         }
@@ -491,15 +368,15 @@ static int vscan_close(struct files_struct *fsp, int fd)
 
 
 	if ( !fsp->modified ) {
-                if ( verbose_file_logging ) 
+                if ( vscan_config.common.verbose_file_logging ) 
                         vscan_syslog("INFO: file %s was not modified - not scanned", filepath);
 
 		return retval;
 	}
 
 	/* don't scan files which are in the list of exclude file types */
-	if ( filetype_skipscan(filepath) == 1 ) {
-                if ( verbose_file_logging )
+	if ( filetype_skipscan(filepath) == VSCAN_FT_SKIP_SCAN ) {
+                if ( vscan_config.common.verbose_file_logging )
                         vscan_syslog("File '%s' not scanned as file type is on exclude list", filepath);
 		return retval;
 	}
@@ -511,12 +388,12 @@ static int vscan_close(struct files_struct *fsp, int fd)
 		/* scan only file, do nothing */
                 rv = vscan_sophos_scanfile(sockfd, filepath, client_ip);
                 vscan_sophos_end(sockfd);
-		if ( rv == 1 ) {
+		if ( rv == VSCAN_SCAN_VIRUS_FOUND ) {
 			/* virus was found */
 #if (SMB_VFS_INTERFACE_VERSION >= 6)
-			rc = vscan_do_infected_file_action(handle, fsp->conn, filepath, quarantine_dir, quarantine_prefix, infected_file_action);
+			rc = vscan_do_infected_file_action(handle, fsp->conn, filepath, vscan_config.common.quarantine_dir, vscan_config.common.quarantine_prefix, vscan_config.common.infected_file_action);
 #else
-			rc = vscan_do_infected_file_action(&default_vfs_ops, fsp->conn, filepath, quarantine_dir, quarantine_prefix, infected_file_action);
+			rc = vscan_do_infected_file_action(&default_vfs_ops, fsp->conn, filepath, vscan_config.common.quarantine_dir, vscan_config.common.quarantine_prefix, vscan_config.common.infected_file_action);
 #endif
 		}
 
